@@ -9,7 +9,7 @@ package Lemonldap::NG::Portal::_Choice;
 use Lemonldap::NG::Portal::Simple;
 use Scalar::Util 'weaken';
 
-our $VERSION = '1.9.3';
+our $VERSION = '1.9.17';
 
 ## @cmethod Lemonldap::NG::Portal::_Choice new(Lemonldap::NG::Portal::Simple portal)
 # Constructor
@@ -59,13 +59,22 @@ sub new {
             $portal->lmLog( "SAML choice force not tested: $@", 'debug' );
         }
 
-        # Force SAML choice if needed
+        # Select SAML choice if needed
         if ($samlForce) {
-            $portal->lmLog( "SAML is a forced choice", 'debug' );
-            $portal->{_authChoice} = 'forcedSAML';
-            $portal->{authChoiceModules}->{'forcedSAML'} = 'SAML;SAML;Null';
+            $portal->lmLog( "Find SAML choice", 'debug' );
+            foreach ( keys %{ $portal->{authChoiceModules} } ) {
+                $portal->{_authChoice} = $_
+                  if ( $portal->{authChoiceModules}->{$_} =~ /^SAML/ );
+            }
+            if ( $portal->{_authChoice} ) {
+                $portal->lmLog(
+                    "SAML Choice " . $portal->{_authChoice} . " found",
+                    'debug' );
+            }
+            else {
+                $portal->lmLog( "No SAML Choice found", 'error' );
+            }
         }
-
     }
 
     return $self unless $portal->{_authChoice};
@@ -179,9 +188,6 @@ sub _buildAuthLoop {
 
         my $name = $_;
 
-        # Ignore 'forcedSAML'
-        next if $name eq 'forcedSAML';
-
         # Name can have a digit as first character
         # for sorting purpose
         # Remove it in displayed name
@@ -191,48 +197,69 @@ sub _buildAuthLoop {
         $name =~ s/\_/ /g;
 
         # Find modules associated to authChoice
-        my ( $auth, $userDB, $passwordDB, $url ) =
+        my ( $auth, $userDB, $passwordDB, $url, $condition ) =
           split( /[;\|]/, $self->{authChoiceModules}->{$_} );
 
-        if ( $auth and $userDB and $passwordDB ) {
+        # Fill sessionInfo to eval rule if empty (unauthenticated user)
+        $self->{sessionInfo}->{_url}   ||= $self->{urldc};
+        $self->{sessionInfo}->{ipAddr} ||= $self->ipAddr;
 
-            # Default URL
-            $url = ( defined $url ? $url .= $ENV{'REQUEST_URI'} : '#' );
-            $self->lmLog( "Use URL $url", 'debug' );
-
-            # Options to store in the loop
-            my $optionsLoop =
-              { name => $name, key => $_, module => $auth, url => $url };
-
-            # Get displayType for this module
-            my $modulePrefix = 'Lemonldap::NG::Portal::';
-            my $authModule   = $modulePrefix . 'Auth' . $auth;
-            $self->loadModule($authModule);
-            my $displayType = &{ $authModule . '::getDisplayType' };
-
-            $self->lmLog( "Display type $displayType for module $auth",
-                'debug' );
-            $optionsLoop->{$displayType} = 1;
-
-            # If displayType is logo, check if key.png is available
-            if (
-                -e $self->getApacheHtdocsPath . "/skins/common/" . $_ . ".png" )
-            {
-                $optionsLoop->{logoFile} = $_ . ".png";
-            }
-            else { $optionsLoop->{logoFile} = $auth . ".png"; }
-
-            # Register item in loop
-            push @authLoop, $optionsLoop;
-
-            $self->lmLog( "Authentication choice $name will be displayed",
-                'debug' );
+        if ( defined $condition and !$self->safe->reval($condition) ) {
+            $self->lmLog(
+"Condition returns false, authentication choice $name will not be displayed",
+                'debug'
+            );
         }
-
         else {
-            $self->abort("Authentication choice $_ value is invalid");
-        }
+            if ( $auth and $userDB and $passwordDB ) {
 
+                # Default URL
+                if ( defined $url
+                    and not $self->checkXSSAttack( 'URI', $ENV{'REQUEST_URI'} )
+                  )
+                {
+                    $url .= $ENV{'REQUEST_URI'};
+                }
+                else {
+                    $url .= '#';
+                }
+                $self->lmLog( "Use URL $url", 'debug' );
+
+                # Options to store in the loop
+                my $optionsLoop =
+                  { name => $name, key => $_, module => $auth, url => $url };
+
+                # Get displayType for this module
+                my $modulePrefix = 'Lemonldap::NG::Portal::';
+                my $authModule   = $modulePrefix . 'Auth' . $auth;
+                $self->loadModule($authModule);
+                my $displayType = &{ $authModule . '::getDisplayType' };
+
+                $self->lmLog( "Display type $displayType for module $auth",
+                    'debug' );
+                $optionsLoop->{$displayType} = 1;
+
+                # If displayType is logo, check if key.png is available
+                if (  -e $self->getApacheHtdocsPath
+                    . "/skins/common/"
+                    . $_
+                    . ".png" )
+                {
+                    $optionsLoop->{logoFile} = $_ . ".png";
+                }
+                else { $optionsLoop->{logoFile} = $auth . ".png"; }
+
+                # Register item in loop
+                push @authLoop, $optionsLoop;
+
+                $self->lmLog( "Authentication choice $name will be displayed",
+                    'debug' );
+
+            }
+            else {
+                $self->abort("Authentication choice $_ value is invalid");
+            }
+        }
     }
 
     return \@authLoop;
